@@ -5,6 +5,12 @@ source $(dirname "$0")/tools.sh
 # Clone NSPR if needed.
 hg_clone https://hg.mozilla.org/projects/nspr ./nspr default
 
+if [[ -f nss/nspr.patch && "$ALLOW_NSPR_PATCH" == "1" ]]; then
+  pushd nspr
+  cat ../nss/nspr.patch | patch -p1
+  popd
+fi
+
 # Build.
 nss/build.sh -g -v --enable-libpkix "$@"
 
