@@ -68,7 +68,7 @@ DigestFile(PLArenaPool *poolp, SECItem ***digests, SECItem *input,
 }
 
 static void
-Usage(char *progName)
+Usage(void)
 {
     fprintf(stderr,
             "Usage:  %s [-C|-D|-E|-O|-S] [<options>] [-d dbdir] [-u certusage]\n"
@@ -115,6 +115,7 @@ Usage(char *progName)
     fprintf(stderr, "%-25s  9 - certUsageProtectedObjectSigner\n", " ");
     fprintf(stderr, "%-25s 10 - certUsageStatusResponder\n", " ");
     fprintf(stderr, "%-25s 11 - certUsageAnyCA\n", " ");
+    fprintf(stderr, "%-25s 12 - certUsageIPsec\n", " ");
 
     exit(-1);
 }
@@ -280,7 +281,6 @@ decode(FILE *out, SECItem *input, const struct decodeOptionsStr *decodeOptions)
                     ** or might be an invalid message, such as a QA test message
                     ** or a message from an attacker.
                     */
-                    SECStatus rv;
                     rv = NSS_CMSSignedData_VerifyCertsOnly(sigd,
                                                            decodeOptions->options->certHandle,
                                                            decodeOptions->options->certUsage);
@@ -1127,7 +1127,7 @@ main(int argc, char **argv)
                     fprintf(stderr,
                             "%s: option -G only supported with option -S.\n",
                             progName);
-                    Usage(progName);
+                    Usage();
                     exit(1);
                 }
                 signOptions.signingTime = PR_TRUE;
@@ -1137,7 +1137,7 @@ main(int argc, char **argv)
                     fprintf(stderr,
                             "%s: option -H only supported with option -S.\n",
                             progName);
-                    Usage(progName);
+                    Usage();
                     exit(1);
                 }
                 decodeOptions.suppressContent = PR_TRUE;
@@ -1167,7 +1167,7 @@ main(int argc, char **argv)
                     fprintf(stderr,
                             "%s: option -N only supported with option -S.\n",
                             progName);
-                    Usage(progName);
+                    Usage();
                     exit(1);
                 }
                 signOptions.nickname = PORT_Strdup(optstate->value);
@@ -1180,7 +1180,7 @@ main(int argc, char **argv)
                     fprintf(stderr,
                             "%s: option -P only supported with option -S.\n",
                             progName);
-                    Usage(progName);
+                    Usage();
                     exit(1);
                 }
                 signOptions.smimeProfile = PR_TRUE;
@@ -1193,7 +1193,7 @@ main(int argc, char **argv)
                     fprintf(stderr,
                             "%s: option -T only supported with option -S.\n",
                             progName);
-                    Usage(progName);
+                    Usage();
                     exit(1);
                 }
                 signOptions.detached = PR_TRUE;
@@ -1203,7 +1203,7 @@ main(int argc, char **argv)
                     fprintf(stderr,
                             "%s: option -Y only supported with option -S.\n",
                             progName);
-                    Usage(progName);
+                    Usage();
                     exit(1);
                 }
                 signOptions.encryptionKeyPreferenceNick = strdup(optstate->value);
@@ -1214,7 +1214,7 @@ main(int argc, char **argv)
                     fprintf(stderr,
                             "%s: option -b only supported with option -D.\n",
                             progName);
-                    Usage(progName);
+                    Usage();
                     exit(1);
                 }
                 batch = PR_TRUE;
@@ -1225,7 +1225,7 @@ main(int argc, char **argv)
                     fprintf(stderr,
                             "%s: option -c only supported with option -D.\n",
                             progName);
-                    Usage(progName);
+                    Usage();
                     exit(1);
                 }
                 contentFile = PR_Open(optstate->value, PR_RDONLY, 006600);
@@ -1261,7 +1261,7 @@ main(int argc, char **argv)
                     fprintf(stderr,
                             "%s: option -h only supported with option -D.\n",
                             progName);
-                    Usage(progName);
+                    Usage();
                     exit(1);
                 }
                 decodeOptions.headerLevel = atoi(optstate->value);
@@ -1288,7 +1288,7 @@ main(int argc, char **argv)
                     fprintf(stderr,
                             "%s: option -k only supported with option -D.\n",
                             progName);
-                    Usage(progName);
+                    Usage();
                     exit(1);
                 }
                 decodeOptions.keepCerts = PR_TRUE;
@@ -1299,7 +1299,7 @@ main(int argc, char **argv)
                     fprintf(stderr,
                             "%s: option -n only supported with option -D.\n",
                             progName);
-                    Usage(progName);
+                    Usage();
                     exit(1);
                 }
                 decodeOptions.suppressContent = PR_TRUE;
@@ -1315,7 +1315,7 @@ main(int argc, char **argv)
             case 'p':
                 if (!optstate->value) {
                     fprintf(stderr, "%s: option -p must have a value.\n", progName);
-                    Usage(progName);
+                    Usage();
                     exit(1);
                 }
 
@@ -1325,7 +1325,7 @@ main(int argc, char **argv)
             case 'f':
                 if (!optstate->value) {
                     fprintf(stderr, "%s: option -f must have a value.\n", progName);
-                    Usage(progName);
+                    Usage();
                     exit(1);
                 }
 
@@ -1335,7 +1335,7 @@ main(int argc, char **argv)
             case 'r':
                 if (!optstate->value) {
                     fprintf(stderr, "%s: option -r must have a value.\n", progName);
-                    Usage(progName);
+                    Usage();
                     exit(1);
                 }
                 envelopeOptions.recipients = ptrarray;
@@ -1368,11 +1368,11 @@ main(int argc, char **argv)
         }
     }
     if (status == PL_OPT_BAD)
-        Usage(progName);
+        Usage();
     PL_DestroyOptState(optstate);
 
     if (mode == UNKNOWN)
-        Usage(progName);
+        Usage();
 
     if (mode != CERTSONLY && !batch) {
         rv = SECU_FileToItem(&input, inFile);
@@ -1529,7 +1529,7 @@ main(int argc, char **argv)
             break;
         default:
             fprintf(stderr, "One of options -D, -S or -E must be set.\n");
-            Usage(progName);
+            Usage();
             exitstatus = 1;
     }
 
